/* PROFESOR.JS version 1.3
   Esta version usa localforage, que usa indexedDB por debajo y por tanto es asincrono
 */

/*---------- OBJETO PROFESOR -------------------------------------------------------

El objeto profesor almacena los datos de un profesor que son:
- Nombre ( obligatorio al crear el objeto ) 
- UUID ( obligatorio al crear el objeto) Es el identificador del profesor
- lunes_e, martes_e, miercoles_e, jueves_e, viernes_e. Horas de entrada de los cinco dias.
- lunes_s, martes_s, miercoles_s, jueves_s, viernes_s. Horas de salida de los cinco dias.
- rueda Titulo de la cadena a la que está adherido

/* Crea un objeto profesor
 * 
 * @param nombreProfe
 * @param numTelefono
 */
function Profesor(nombreProfe, UUID, l_e, l_s, m_e, m_s, x_e, x_s, j_e, j_s, v_e, v_s, nuevaRueda, idBD) {

    this.nombre = nombreProfe;
    this.UUID = UUID;

    // Si no nos proporcionan horario ponemos por defecto [1,1,1,1,1] y [6,6,6,6,6]
    this.lunes_e = (l_e === undefined ? 1 : l_e);
    this.martes_e = (m_e === undefined ? 1 : m_e);
    this.miercoles_e = (x_e === undefined ? 1 : x_e);
    this.jueves_e = (j_e === undefined ? 1 : j_e);
    this.viernes_e = (v_e === undefined ? 1 : v_e);
    this.lunes_s = (l_s === undefined ? 6 : l_s);
    this.martes_s = (m_s === undefined ? 6 : m_s);
    this.miercoles_s = (x_s === undefined ? 6 : x_s);
    this.jueves_s = (j_s === undefined ? 6 : j_s);
    this.viernes_s = (v_s === undefined ? 6 : v_s);

    // Si no nos proporcionan rueda ponemos "ninguna"
    this.rueda = (nuevaRueda === undefined ? "ninguna" : nuevaRueda);

    // Identificador del registro en la BD
    this.idBD = (nuevaRueda === undefined ? "-1" : idBD);
}

/* Añade un horario al profesor
 * @param l_e Numero que indica la hora de entrada los lunes
 * @param l_s Numero que indica la hora de salida los lunes
 * @param el resto de parametros son los homologos a los dos anteriores para el resto de los dias
 */
Profesor.prototype.addHorario = function(l_e, l_s, m_e, m_s, x_e, x_s, j_e, j_s, v_e, v_s) {
    this.lunes_e = l_e;
    this.martes_e = m_e;
    this.miercoles_e = x_e;
    this.jueves_e = j_e;
    this.viernes_e = v_e;
    this.lunes_s = l_s;
    this.martes_s = m_s;
    this.miercoles_s = x_s;
    this.jueves_s = j_s;
    this.viernes_s = v_s;
}

Profesor.prototype.getNombre = function() {
    return this.nombre;
}

Profesor.prototype.getUUID = function() {
    return this.UUID;
}

Profesor.prototype.getRueda = function() {
    return this.rueda;
}

Profesor.prototype.getIDBD = function() {
    return this.idBD;
}

Profesor.prototype.setRueda = function(nuevaRueda) {
    this.rueda = nuevaRueda;
}

Profesor.prototype.setIDBD = function(idBD) {
    this.idBD = idBD;
}

/* Devuelve la hora de salida de un dia especifico de la semana
 * @param dia Dia de la semana: lunes(0), martes(1), miercoles(2), jueves(3) o viernes(4) 
 */
Profesor.prototype.getSalida = function(dia) {
    switch (dia) {
        case 0:
            return this.lunes_s;
        case 1:
            return this.martes_s;
        case 2:
            return this.miercoles_s;
        case 3:
            return this.jueves_s;
        case 4:
            return this.viernes_s;
    }
}

/* Devuelve la hora de entrada de un dia especifico de la semana
 * @param dia Dia de la semana: lunes(0), martes(1), miercoles(2), jueves(3) o viernes(4) 
 */
Profesor.prototype.getEntrada = function(dia) {
    switch (dia) {
        case 0:
            return this.lunes_e;
        case 1:
            return this.martes_e;
        case 2:
            return this.miercoles_e;
        case 3:
            return this.jueves_e;
        case 4:
            return this.viernes_e;
    }
}


/* Recupera un objeto Profesor del almacenamiento local
 *
 * @param numTelefono:entero Numero de telefono del profesor
 */

Profesor.recuperaProfesor = async function(UUID) {
    var profesor = null;

    try {
        var profeJSON = await localforage.getItem(UUID);

        if (profeJSON != null) {
            
            profesor = new Profesor(profeJSON.nombre,profeJSON.UUID,profeJSON.lunes_e,profeJSON.lunes_s,profeJSON.martes_e,
                profeJSON.martes_s,profeJSON.miercoles_e,profeJSON.miercoles_s,profeJSON.jueves_e,profeJSON.jueves_s,
                profeJSON.viernes_e,profeJSON.viernes_s,profeJSON.rueda,profeJSON.idBD);
        }

    } catch (err) {
        alert("No es posible recuperar el usuario del almacenamiento local: " + err.message);
    }

    return profesor;
}

/* Almacena el usuario en el almacenamiento local del navegador
 */
Profesor.prototype.almacenarLocal = function() {

    try {
        localforage.setItem(this.UUID, this);
    } catch (err) {
        alert("No es posible almacenar el usuario: " + err.message);
    }
}

/* Almacena el usuario en el servidor por primera vez y actualiza el atributo idBD
 * con el identificador del registro asignado por la BD.
 */
Profesor.prototype.almacenarServer = function() {

    var settings = {
        "async": true,
        "crossDomain": true,
        "dataType": "json",
        "url": "https://ruedacoches-e561.restdb.io/rest/profesor",
        "method": "POST",
        "headers": {
            "content-type": "application/json",
            "x-apikey": "5ae6c28825a622ae4d5286fd",
            "cache-control": "no-cache"
        },
        "processData": false,
        "context": this,
        "data": JSON.stringify(this),
    }

    $.ajax(settings).done(function(response) {
        this.setIDBD(response._id);
        this.almacenarLocal();
    });

}

/* Actualiza el usuario en el servidor
 */
Profesor.prototype.actualizaServer = function() {

    var settings = {
        "async": true,
        "crossDomain": true,
        "dataType": "json",
        "url": "https://ruedacoches-e561.restdb.io/rest/profesor/" + this.getIDBD(),
        "method": "PUT",
        "headers": {
            "content-type": "application/json",
            "x-apikey": "5ae6c28825a622ae4d5286fd",
            "cache-control": "no-cache"
        },
        "processData": false,
        "data": JSON.stringify(this),

        success: function(data, status, jqxhr) {
            console.log("Request received:", data);
        },
        error: function(jqxhr, status, error) {
            console.log("Something went wrong!" + jqxhr + " ---" + status + " ---" + error);
        }
    }

    $.ajax(settings).done(function(response) {});
}


/* Recupera un profesor del servidor
 * @param numTelefono: numerico. El numero de telefono es el identificador del usuario
 *
 * return Profesor.
 */
Profesor.recuperaServer = function(UUID) {

    return new Promise(function(resolve, reject) {
        var settings = {
            'async': true,
            'crossDomain': true,
            'dataType': 'json',
            'url': 'https://ruedacoches-e561.restdb.io/rest/profesor?q={"UUID": "' + UUID + '"}',
            'method': 'GET',
            'headers': {
                'content-type': 'application/json',
                'x-apikey': '5ae6c28825a622ae4d5286fd',
                'cache-control': 'no-cache'
            }
        }

        $.ajax(settings).done(function(response) {
                            
            if (!$.trim(response)) { // Comprobar que response es afirmativa 
                resolve(null);
            } else {
                resolve(new Profesor(
                    response[0].nombre, response[0].UUID, 
                    response[0].lunes_e, response[0].lunes_s,
                    response[0].martes_e, response[0].martes_s,
                    response[0].miercoles_e, response[0].miercoles_s,
                    response[0].jueves_e, response[0].jueves_s,
                    response[0].viernes_e, response[0].viernes_s,
                    response[0].rueda, response[0]._id));

            }
        });

    });
}