/* RUEDA.JS version 1.2
    Esta version usa localforage, que usa indexedDB por debajo y por tanto es asincrono
 */

/* Constantes utiles 
 */
Rueda.DIA_LETRAS = ["L", "M", "X", "J", "V", "S", "D"];
Rueda.DIA_NOMBRES = ["Lunes", "Martes", "Miercoles", "Jueves", "Viernes", "Sabado", "Domingo"];

/*---------- OBJETO RUEDA -------------------------------------------------------*/

/* Crea un objeto rueda nuevo
 *
 * @param IDRueda 
 */
function Rueda(propietario, titulo) {

    this.titulo = (titulo === undefined ? '' : titulo);
    this.miembros = new Array(); // Datos de los profesores
    this.propietario = propietario; // Profesor propietario del dispositivo. La rueda se calcula en torno a él

}

Rueda.prototype.setTitulo = function(tituloRueda) {
    this.titulo = tituloRueda;
}


Rueda.prototype.setPropietario = function(propietario) {
    this.propietario = propietario;
}

Rueda.prototype.getTitulo = function() {
    return this.titulo;
}

/* La rueda esta vacia si no tiene miembros
 */
Rueda.prototype.esVacia = function() {
    return this.miembros.length == 0;
}

/* Añade o modifica un miembro a la rueda.
 *
 * @param nombre_n Nombre del nuevo miembro   
 * @param horario_n Array con el horario del nuevo profesor del tipo: [1,6,1,4,2,6,3,6,1,6]
 */
Rueda.prototype.addMiembro = function(nuevoProfe) {
    this.miembros.push(nuevoProfe);
}

/* Recupera un profesor de la rueda o NULL si no lo encuentra
 *
 * @param numTlf:num Numero de telefono del profesor a recuperar   
 *
 * Return Profesor
 */
Rueda.prototype.getMiembro = function(numTlf) {

    for (var i = 0; i < this.miembros.length; i++)
        if (this.miembros[i].getNumTlf() == numTlf) return this.miembros[i];

    return null;
}

/* Elimina un miembro de la rueda.
 *
 * @param nombre_n Nombre del nuevo miembro   
 */
Rueda.prototype.delMiembro = function(profe) {
    var borrar = -1;

    if (!this.esVacia()) {

        for (i = 0; i < this.miembros.length; i++) {
            if (this.miembros[i].getUUID() == profe.getUUID()) {
                borrar = i;
            }
        }
        if (borrar > -1) {
            this.miembros.splice(borrar, 1);
        }
    }

}

/* Actualiza miembro
 */
Rueda.prototype.actualizaMiembro = function(profe) {

    if (!this.esVacia()) {

        for (i = 0; i < this.miembros.length; i++) {
            if (this.miembros[i].getUUID() == profe.getUUID()) {
                this.miembros[i] = profe;
            }
        }

    }

}

/* Guarda la rueda en el almacenamiento local
 */
Rueda.prototype.almacenar = function() {

    try {
        localforage.setItem("rueda", this);
    } catch (err) {
        alert("No es posible almacenar la rueda: " + err.message);
    }
}

/* Guarda la rueda en el almacenamiento local
 */
Rueda.prototype.borrar = function(IDRueda) {

    try {
        localforage.removeItem(IDRueda);
    } catch (err) {
        alert("No es posible eliminar la rueda: " + err.message);
    }
}

/* Recupera la rueda del almacenamiento local
 *
 * @param IDRueda:string 
 */
Rueda.recuperaRueda = async function(IDRueda) {
    var rueda = null;

    try {
        var datosRueda = await localforage.getItem("rueda");

        if (datosRueda != null) {

//            var datosRueda = JSON.parse(ruedaJSON);
            var propi = new Profesor(datosRueda.propietario.nombre, datosRueda.propietario.UUID,
                datosRueda.propietario.lunes_e, datosRueda.propietario.lunes_s,
                datosRueda.propietario.martes_e, datosRueda.propietario.martes_s,
                datosRueda.propietario.miercoles_e, datosRueda.propietario.miercoles_s,
                datosRueda.propietario.jueves_e, datosRueda.propietario.jueves_s,
                datosRueda.propietario.viernes_e, datosRueda.propietario.viernes_s,
                datosRueda.propietario.rueda);

            rueda = new Rueda(propi, datosRueda.titulo);

            datosRueda.miembros.forEach(function(profe) {
                rueda.addMiembro(
                    new Profesor(
                        profe.nombre, profe.UUID, profe.lunes_e, profe.lunes_s, profe.martes_e,
                        profe.martes_s, profe.miercoles_e, profe.miercoles_s, profe.jueves_e,
                        profe.jueves_s, profe.viernes_e, profe.viernes_s, profe.rueda));

            });

        }

    } catch (err) {
        alert("No es posible recuperar la rueda del almacenamiento local: " + err.message);
    }
    return rueda;
}


Rueda.existe = function(tituloRueda) {

    return new Promise(function(resolve, reject) {
        var settings = {
            'async': true,
            'crossDomain': true,
            'dataType': 'json',
            'url': 'https://ruedacoches-e561.restdb.io/rest/profesor?q={"rueda": "' + tituloRueda + '"}',
            'method': 'GET',
            'headers': {
                'content-type': 'application/json',
                'x-apikey': '5ae6c28825a622ae4d5286fd',
                'cache-control': 'no-cache'
            }
        }

        $.ajax(settings).done(function(response) {

            if (!$.trim(response)) {
                resolve(false);
            } else {
                resolve(true);
            }

        });

    });
}


/* Recupera una rueda del servidor
 *
 * return Profesor.
 */
Rueda.recuperaRuedaServer = function(tituloRueda) {

    return new Promise(function(resolve, reject) {
        var settings = {
            'async': true,
            'crossDomain': true,
            'dataType': 'json',
            'url': 'https://ruedacoches-e561.restdb.io/rest/profesor?q={"rueda": "' + tituloRueda + '"}',
            'method': 'GET',
            'headers': {
                'content-type': 'application/json',
                'x-apikey': '5ae6c28825a622ae4d5286fd',
                'cache-control': 'no-cache'
            }
        }

        $.ajax(settings).done(function(response) {

            if (1 == 1) {
                var rueda = new Rueda();
                response.forEach(function(profe) {
                    rueda.addMiembro(
                        new Profesor(
                            profe.nombre, profe.UUID, profe.lunes_e, profe.lunes_s, profe.martes_e,
                            profe.martes_s, profe.miercoles_e, profe.miercoles_s, profe.jueves_e,
                            profe.jueves_s, profe.viernes_e, profe.viernes_s, profe.rueda, profe._id));
                    rueda.setTitulo(tituloRueda);
                });
                resolve(rueda);
            } else {
                reject(Error("El usuario no está registrado en el servidor"));
            }
        });

    });
}


/* Retorna una cadena de caracteres con los IDs de los profesores
   que componen esta rueda separados por comas
 */
function IDProfesores() {
    var IDs = '';

    for (var i = 0; i < this.miembros.length; i++) {
        IDs += this.miembros[i].getID() + ",";
    }

    return IDs.slice(0, -1);
}

/* Version 1.0.
 * Asume que:
 * - los profesores solo pueden ir y venir en el mismo coche. Con lo que se debe 
 *   buscar la coincidencia de la entrada y la salida.
 * - El punto de partida es el mismo para todos. Con lo que no importa si van varios coches,
 *   los profesores se distribuyen después a su antojo.
 *
 * Genera la rueda para este usuario un dia de la semana.
 * La rueda es diferente para cada usuario porque solo les importa los horarios de las 
 * personas con las que coinciden. Por tanto solo se buscan las coincidencias con el 
 * usuario actua.
 * 
 * @param dia:numero Dia de la semana: lunes(0), martes(1), miercoles(2), jueves(3) o viernes(4) 
 * @param usuario:Profesor Profesor para el que buscamos la coincidencia
 *
 * return Un array con los profesores que coinciden, ordenados por numero de telefono
 */
Rueda.prototype.generarRuedaDia = function(dia) {

    var coincidencias = new Array(); // Contiene una cadena con los nombres de los profesores que coinciden este dia

    // Recorremos la rueda comprobando si los profesores coinciden en la entrada y salida en este dia
    for (var i = 0; i < this.miembros.length; i++) {
        if (this.propietario.getEntrada(dia) == this.miembros[i].getEntrada(dia) &&
            this.propietario.getSalida(dia) == this.miembros[i].getSalida(dia)) {
            coincidencias.push(this.miembros[i]);
        }

    }

    // Se ordena por numero de ID. 
    coincidencias.sort(function(a, b) {
        if (a.getUUID() < b.getUUID()) return -1;
        if (a.getUUID() > b.getUUID()) return 1;
        return 0;
    });

    return coincidencias;
}



/* Crea un array con las coincidencias de una semana
 * Hace uso de la versión 1.0 generarRuedaDia heredando su limitaciones.
 * 
 * @param usuario:Profesor El profesor para el que buscamos la coincidencia
 *
 * return Un array con la lista de las coincidencias de toda la semana.
 */
Rueda.prototype.generarRuedaSemana = function() {
    var semana = new Array();

    for (var i = 0; i < 5; i++)
        semana.push(this.generarRuedaDia(i));

    return semana;
}

/* Crea un calendario para el curso donde se indica quien conduce cada dia 
 * Hace uso de generarRuedaDia heredando sus limitaciones
 */
Rueda.prototype.conductoresCurso = function() {
    var calendario = new Array();

    var lunes = this.generarRuedaDia(0);
    var martes = this.generarRuedaDia(1);
    var miercoles = this.generarRuedaDia(2);
    var jueves = this.generarRuedaDia(3);
    var viernes = this.generarRuedaDia(4);

    for (var i = 0; i < 43 * 5; i += 5) { // El curso tiene 43 semanas de 5 dias

        calendario[i] = lunes[(i / 5) % lunes.length].getUUID();
        calendario[i + 1] = martes[(i / 5) % martes.length].getUUID();
        calendario[i + 2] = miercoles[(i / 5) % miercoles.length].getUUID();
        calendario[i + 3] = jueves[(i / 5) % jueves.length].getUUID();
        calendario[i + 4] = viernes[(i / 5) % viernes.length].getUUID();

    }

    return calendario;
}

Rueda.prototype.conductoresSemana = function(semana, conductores) {
    var conductorSemana = new Array(); // Array con los identificadores de los conductores de la semana

    for (var i = 0; i < 5; i++)
        conductorSemana[i] = conductores[(semana * 5) + i];

    return conductorSemana;

}

/* Devuelve el primer lunes de septiembre, que es cuando empezaremos a contar las 43 
 * semanas del curso
 */
Rueda.primerLunesSeptiembre = function() {

    // De septiembre a diciembre el inicio de curso es el año actual.
    // De enero a junio el inicio del curso es el año anterior
    var hoy = new Date();
    var year = hoy.getMonth() > 8 ? hoy.getFullYear() : hoy.getFullYear() - 1;

    var lunes = new Date(year, 8, 1, 0, 0, 0, 0);

    if (lunes.getDay() != 1) {

        if (lunes.getDay() == 0) // Si es domingo, el segundo dia del mes es lunes
            lunes = lunes.setDate(2);
        else
            lunes = lunes.setDate(9 - lunes.getDay());

    }

    return new Date(lunes);

}


Rueda.lunesSemana = function(sem){

    var lunes = Rueda.primerLunesSeptiembre();
    lunes.setDate(lunes.getDate()+(sem*7));
    return lunes;

}

Rueda.domingoSemana = function(sem){

    var lunes = Rueda.primerLunesSeptiembre();
    lunes.setDate(lunes.getDate()+6+(sem*7));
    return lunes;

}

/* Devuelve la semana en curso encuadrado dentro de los dias de las 43 semanas del curso
 */
Rueda.semanaEnCurso = function() {

    var hoy = new Date();
    var primerLunes = Rueda.primerLunesSeptiembre();

    // Diferencia de fechas en semanas
    var difSemanas = Math.floor((hoy.getTime() - primerLunes.getTime()) / 604800000);

    return difSemanas;
}